@@ -257,7 +257,7 @@ class S3FileSystem(AsyncFileSystem):
     retries = 5
     read_timeout = 15
     default_block_size = 5 * 2**20
-    protocol = ["s3", "s3a"]
+    protocol = ("s3", "s3a")
     _extra_tokenize_attributes = ("default_block_size",)
 
     def __init__(
@@ -541,6 +541,12 @@ class S3FileSystem(AsyncFileSystem):
     @staticmethod
     def close_session(loop, s3):
         if loop is not None and loop.is_running():
+            try:
+                loop = asyncio.get_event_loop()
+                loop.create_task(s3.__aexit__(None, None, None))
+                return
+            except RuntimeError:
+                pass
             try:
                 sync(loop, s3.__aexit__, None, None, None, timeout=0.1)
                 return
@@ -593,11 +599,12 @@ class S3FileSystem(AsyncFileSystem):
         path,
         mode="rb",
         block_size=None,
-        acl="",
+        acl=False,
         version_id=None,
         fill_cache=None,
         cache_type=None,
         autocommit=True,
+        size=None,
         requester_pays=None,
         cache_options=None,
         **kwargs,
@@ -619,7 +626,8 @@ class S3FileSystem(AsyncFileSystem):
             best support random access. When reading only a few specific chunks
             out of a file, performance may be better if False.
         acl: str
-            Canned ACL to set when writing
+            Canned ACL to set when writing. False sends no parameter and uses the bucket's
+            preset default; otherwise it should be a member of the `key_acls` set.
         version_id : str
             Explicit version of the object to open.  This requires that the s3
             filesystem is version aware and bucket versioning is enabled on the
@@ -644,7 +652,11 @@ class S3FileSystem(AsyncFileSystem):
         if requester_pays is None:
             requester_pays = bool(self.req_kw)
 
-        acl = acl or self.s3_additional_kwargs.get("ACL", "")
+        acl = (
+            acl
+            or self.s3_additional_kwargs.get("ACL", False)
+            or self.s3_additional_kwargs.get("acl", False)
+        )
         kw = self.s3_additional_kwargs.copy()
         kw.update(kwargs)
         if not self.version_aware and version_id:
@@ -669,6 +681,7 @@ class S3FileSystem(AsyncFileSystem):
             autocommit=autocommit,
             requester_pays=requester_pays,
             cache_options=cache_options,
+            size=size,
         )
 
     async def _lsdir(
@@ -840,6 +853,13 @@ class S3FileSystem(AsyncFileSystem):
                         thisdircache[ppar].append(d)
             if par in sdirs:
                 thisdircache[par].append(o)
+
+        # Explicitly add directories to their parents in the dircache
+        for d in dirs:
+            par = self._parent(d["name"])
+            if par in thisdircache:
+                thisdircache[par].append(d)
+
         if not prefix:
             for k, v in thisdircache.items():
                 if k not in self.dircache and len(k) >= len(path):
@@ -852,7 +872,7 @@ class S3FileSystem(AsyncFileSystem):
 
     find = sync_wrapper(_find)
 
-    async def _mkdir(self, path, acl="", create_parents=True, **kwargs):
+    async def _mkdir(self, path, acl=False, create_parents=True, **kwargs):
         path = self._strip_protocol(path).rstrip("/")
         if not path:
             raise ValueError
@@ -866,7 +886,9 @@ class S3FileSystem(AsyncFileSystem):
             if acl and acl not in buck_acls:
                 raise ValueError("ACL not in %s", buck_acls)
             try:
-                params = {"Bucket": bucket, "ACL": acl}
+                params = {"Bucket": bucket}
+                if acl:
+                    params["ACL"] = acl
                 region_name = kwargs.get("region_name", None) or self.client_kwargs.get(
                     "region_name", None
                 )
@@ -899,6 +921,13 @@ class S3FileSystem(AsyncFileSystem):
     makedirs = sync_wrapper(_makedirs)
 
     async def _rmdir(self, path):
+        bucket, key, _ = self.split_path(path)
+        if key:
+            if await self._exists(path):
+                # User may have meant rm(path, recursive=True)
+                raise FileExistsError
+            raise FileNotFoundError
+
         try:
             await self._call_s3("delete_bucket", Bucket=path)
         except botocore.exceptions.ClientError as e:
@@ -963,6 +992,8 @@ class S3FileSystem(AsyncFileSystem):
                     for o in files
                     if o["name"].rstrip("/") == path and o["type"] != "directory"
                 ]
+                if not files:
+                    raise FileNotFoundError(path)
             if detail:
                 return files
         return files if detail else sorted([o["name"] for o in files])
@@ -1186,7 +1217,7 @@ class S3FileSystem(AsyncFileSystem):
                 Bucket=bucket,
                 Key=key,
                 **version_id_kw(version_id or vers),
-                **self.req_kw,
+                **kw,
             )
             return resp["Body"], resp.get("ContentLength", None)
 
@@ -1217,10 +1248,10 @@ class S3FileSystem(AsyncFileSystem):
                         # in a failure.
                         # Examples:
                         # Read 1 byte -> failure, retry with read_range=0, byte range should be 0-
-                        # Read 1 byte, success. Read 1 byte: failure. Retry with read_range=2, byte-range should be 2-
-                        # Read 1 bytes, success. Read 1 bytes: success. Read 1 byte, failure. Retry with read_range=3,
-                        # byte-range should be 3-.
-                        body, _ = await _open_file(bytes_read + 1)
+                        # Read 1 byte, success. Read 1 byte: failure. Retry with read_range=1, byte-range should be 1-
+                        # Read 1 bytes, success. Read 1 bytes: success. Read 1 byte, failure. Retry with read_range=2,
+                        # byte-range should be 2-.
+                        body, _ = await _open_file(bytes_read)
                         continue
 
                     if not chunk:
@@ -1262,7 +1293,7 @@ class S3FileSystem(AsyncFileSystem):
                     if out:
                         return out[0]
                 else:
-                    out = [o for o in out if o["name"] == path]
+                    out = [o for o in out if o["name"] == fullpath]
                     if out:
                         return out[0]
                     return {"name": path, "size": 0, "type": "directory"}
@@ -1278,7 +1309,7 @@ class S3FileSystem(AsyncFileSystem):
                 )
                 return {
                     "ETag": out.get("ETag", ""),
-                    "LastModified": out["LastModified"],
+                    "LastModified": out.get("LastModified", ""),
                     "size": out["ContentLength"],
                     "name": "/".join([bucket, key]),
                     "type": "file",
@@ -1873,6 +1904,7 @@ class S3FileSystem(AsyncFileSystem):
 
     async def _rm_file(self, path, **kwargs):
         bucket, key, _ = self.split_path(path)
+        self.invalidate_cache(path)
 
         try:
             await self._call_s3("delete_object", Bucket=bucket, Key=key)
@@ -1911,6 +1943,17 @@ class S3FileSystem(AsyncFileSystem):
 
     is_bucket_versioned = sync_wrapper(_is_bucket_versioned)
 
+    async def _make_bucket_versioned(self, bucket, versioned: bool = True):
+        """Set bucket versioning status"""
+        status = "Enabled" if versioned else "Suspended"
+        return await self._call_s3(
+            "put_bucket_versioning",
+            Bucket=bucket,
+            VersioningConfiguration={"Status": status},
+        )
+
+    make_bucket_versioned = sync_wrapper(_make_bucket_versioned)
+
     async def _rm_versioned_bucket_contents(self, bucket):
         """Remove a versioned bucket and all contents"""
         await self.set_session()
@@ -1949,7 +1992,7 @@ class S3FileSystem(AsyncFileSystem):
         if "LastModified" not in info:
             # This path is a bucket or folder, which do not currently have a modified date
             raise IsADirectoryError
-        return info["LastModified"].replace(tzinfo=None)
+        return info["LastModified"]
 
     def sign(self, path, expiration=100, **kwargs):
         return self.url(path, expires=expiration, **kwargs)
@@ -2025,7 +2068,7 @@ class S3File(AbstractBufferedFile):
         path,
         mode="rb",
         block_size=5 * 2**20,
-        acl="",
+        acl=False,
         version_id=None,
         fill_cache=True,
         s3_additional_kwargs=None,
@@ -2033,6 +2076,7 @@ class S3File(AbstractBufferedFile):
         cache_type="readahead",
         requester_pays=False,
         cache_options=None,
+        size=None,
     ):
         bucket, key, path_version_id = s3.split_path(path)
         if not key:
@@ -2071,6 +2115,7 @@ class S3File(AbstractBufferedFile):
             autocommit=autocommit,
             cache_type=cache_type,
             cache_options=cache_options,
+            size=size,
         )
         self.s3 = self.fs  # compatibility
 
@@ -2110,7 +2155,7 @@ class S3File(AbstractBufferedFile):
             # Reflect head
             self.s3_additional_kwargs.update(head)
 
-        if "r" in mode and "ETag" in self.details:
+        if "r" in mode and size is None and "ETag" in self.details:
             self.req_kw["IfMatch"] = self.details["ETag"]
 
     def _call_s3(self, method, *kwarglist, **kwargs):
@@ -2122,12 +2167,13 @@ class S3File(AbstractBufferedFile):
             return
         logger.debug("Initiate upload for %s" % self)
         self.parts = []
-        self.mpu = self._call_s3(
-            "create_multipart_upload",
+        kw = dict(
             Bucket=self.bucket,
             Key=self.key,
-            ACL=self.acl,
         )
+        if self.acl:
+            kw["ACL"] = self.acl
+        self.mpu = self._call_s3("create_multipart_upload", **kw)
 
         if self.append_block:
             # use existing data in key when appending,
@@ -2244,7 +2290,10 @@ class S3File(AbstractBufferedFile):
                 Key=key,
             )
 
-            self.parts.append({"PartNumber": part, "ETag": out["ETag"]})
+            part_header = {"PartNumber": part, "ETag": out["ETag"]}
+            if "ChecksumSHA256" in out:
+                part_header["ChecksumSHA256"] = out["ChecksumSHA256"]
+            self.parts.append(part_header)
 
         if self.autocommit and final:
             self.commit()
@@ -2256,20 +2305,16 @@ class S3File(AbstractBufferedFile):
             if self.buffer is not None:
                 logger.debug("Empty file committed %s" % self)
                 self._abort_mpu()
-                write_result = self.fs.touch(self.path)
+                write_result = self.fs.touch(self.path, **self.kwargs)
         elif not self.parts:
             if self.buffer is not None:
                 logger.debug("One-shot upload of %s" % self)
                 self.buffer.seek(0)
                 data = self.buffer.read()
-                write_result = self._call_s3(
-                    "put_object",
-                    Key=self.key,
-                    Bucket=self.bucket,
-                    Body=data,
-                    ACL=self.acl,
-                    **self.kwargs,
-                )
+                kw = dict(Key=self.key, Bucket=self.bucket, Body=data, **self.kwargs)
+                if self.acl:
+                    kw["ACL"] = self.acl
+                write_result = self._call_s3("put_object", **kw)
             else:
                 raise RuntimeError
         else:
@@ -2349,12 +2394,18 @@ def _fetch_range(fs, bucket, key, version_id, start, end, req_kw=None):
 
 
 async def _inner_fetch(fs, bucket, key, version_id, start, end, req_kw=None):
-    resp = await fs._call_s3(
-        "get_object",
-        Bucket=bucket,
-        Key=key,
-        Range="bytes=%i-%i" % (start, end - 1),
-        **version_id_kw(version_id),
-        **req_kw,
-    )
-    return await resp["Body"].read()
+    async def _call_and_read():
+        resp = await fs._call_s3(
+            "get_object",
+            Bucket=bucket,
+            Key=key,
+            Range="bytes=%i-%i" % (start, end - 1),
+            **version_id_kw(version_id),
+            **req_kw,
+        )
+        try:
+            return await resp["Body"].read()
+        finally:
+            resp["Body"].close()
+
+    return await _error_wrapper(_call_and_read, retries=fs.retries)
