@@ -13,6 +13,7 @@ import time
 import sys
 import pytest
 import moto
+from moto.moto_server.threaded_moto_server import ThreadedMotoServer
 from itertools import chain
 import fsspec.core
 from dateutil.tz import tzutc
@@ -67,47 +68,29 @@ port = 5555
 endpoint_uri = "http://127.0.0.1:%s/" % port
 
 
-@pytest.fixture()
+@pytest.fixture(scope="module")
 def s3_base():
     # writable local S3 system
-    import shlex
-    import subprocess
 
-    try:
-        # should fail since we didn't start server yet
-        r = requests.get(endpoint_uri)
-    except:
-        pass
-    else:
-        if r.ok:
-            raise RuntimeError("moto server already up")
+    # This fixture is module-scoped, meaning that we can re-use the MotoServer across all tests
+    server = ThreadedMotoServer(ip_address="127.0.0.1", port=port)
+    server.start()
     if "AWS_SECRET_ACCESS_KEY" not in os.environ:
         os.environ["AWS_SECRET_ACCESS_KEY"] = "foo"
     if "AWS_ACCESS_KEY_ID" not in os.environ:
         os.environ["AWS_ACCESS_KEY_ID"] = "foo"
-    proc = subprocess.Popen(
-        shlex.split("moto_server s3 -p %s" % port),
-        stderr=subprocess.DEVNULL,
-        stdout=subprocess.DEVNULL,
-        stdin=subprocess.DEVNULL,
-    )
 
-    timeout = 5
-    while timeout > 0:
-        try:
-            print("polling for moto server")
-            r = requests.get(endpoint_uri)
-            if r.ok:
-                break
-        except:
-            pass
-        timeout -= 0.1
-        time.sleep(0.1)
     print("server up")
     yield
     print("moto done")
-    proc.terminate()
-    proc.wait()
+    server.stop()
+
+
+@pytest.fixture(autouse=True)
+def reset_s3_fixture():
+    # We reuse the MotoServer for all tests
+    # But we do want a clean state for every test
+    requests.post(f"{endpoint_uri}/moto-api/reset")
 
 
 def get_boto3_client():
@@ -183,6 +166,18 @@ def test_simple(s3):
         assert out == data
 
 
+def test_with_size(s3):
+    data = b"a" * (10 * 2**20)
+
+    with s3.open(a, "wb") as f:
+        f.write(data)
+
+    with s3.open(a, "rb", size=100) as f:
+        assert f.size == 100
+        out = f.read()
+        assert len(out) == 100
+
+
 @pytest.mark.parametrize("default_cache_type", ["none", "bytes", "mmap", "readahead"])
 def test_default_cache_type(s3, default_cache_type):
     data = b"a" * (10 * 2**20)
@@ -276,12 +271,14 @@ def test_info(s3):
     s3.ls(parent)  # fill the cache with parent dir
     assert s3.info(a) == s3.dircache[parent][0]  # correct value
     assert id(s3.info(a)) == id(s3.dircache[parent][0])  # is object from cache
+    assert id(s3.info(f"/{a}")) == id(s3.dircache[parent][0])  # is object from cache
 
     new_parent = test_bucket_name + "/foo"
     s3.mkdir(new_parent)
     with pytest.raises(FileNotFoundError):
         s3.info(new_parent)
-    s3.ls(new_parent)
+    with pytest.raises(FileNotFoundError):
+        s3.ls(new_parent)
     with pytest.raises(FileNotFoundError):
         s3.info(new_parent)
 
@@ -374,6 +371,35 @@ def test_checksum(s3):
     s3.checksum(path1, refresh=True)
 
 
+def test_multi_checksum(s3):
+    # Moto accepts the request to add checksum, and accepts the checksum mode,
+    # but doesn't actually return the checksum
+    # So, this is mostly a stub test
+    file_key = "checksum"
+    path = test_bucket_name + "/" + file_key
+    s3 = S3FileSystem(
+        anon=False,
+        client_kwargs={"endpoint_url": endpoint_uri},
+        s3_additional_kwargs={"ChecksumAlgorithm": "SHA256"},
+    )
+    with s3.open(
+        path,
+        "wb",
+        blocksize=5 * 2**20,
+    ) as f:
+        f.write(b"0" * (5 * 2**20 + 1))  # starts multipart and puts first part
+        f.write(b"data")  # any extra data
+    assert s3.cat(path) == b"0" * (5 * 2**20 + 1) + b"data"
+    FileHead = sync(
+        s3.loop,
+        s3.s3.head_object,
+        Bucket=test_bucket_name,
+        Key=file_key,
+        ChecksumMode="ENABLED",
+    )
+    # assert "ChecksumSHA256" in FileHead
+
+
 test_xattr_sample_metadata = {"testxattr": "1"}
 
 
@@ -591,6 +617,22 @@ def test_rmdir(s3):
     s3.rmdir(bucket)
     assert bucket not in s3.ls("/")
 
+    # Issue 689, s3fs rmdir command returns error when given a valid s3 path.
+    dir = test_bucket_name + "/dir"
+
+    assert not s3.exists(dir)
+    with pytest.raises(FileNotFoundError):
+        s3.rmdir(dir)
+
+    s3.touch(dir + "/file")
+    assert s3.exists(dir)
+    assert s3.exists(dir + "/file")
+    with pytest.raises(FileExistsError):
+        s3.rmdir(dir)
+
+    with pytest.raises(OSError):
+        s3.rmdir(test_bucket_name)
+
 
 def test_mkdir(s3):
     bucket = "test1_bucket"
@@ -742,6 +784,12 @@ def test_du(s3):
     )
     assert s3.du(test_bucket_name) == s3.du("s3://" + test_bucket_name)
 
+    # Issue 450, s3.du of non-existent directory
+    dir = test_bucket_name + "/does-not-exist"
+    assert not s3.exists(dir)
+    assert s3.du(dir) == 0
+    assert s3.du(dir + "/") == 0
+
 
 def test_s3_ls(s3):
     fn = test_bucket_name + "/nested/file1"
@@ -1986,6 +2034,7 @@ def test_modified(s3):
     s3.touch(file_path)
     modified = s3.modified(path=file_path)
     assert isinstance(modified, datetime.datetime)
+    assert modified.tzinfo is not None
 
     # Test directory
     with pytest.raises(IsADirectoryError):
@@ -2070,9 +2119,13 @@ def test_via_fsspec(s3):
     import fsspec
 
     s3.mkdir("mine")
-    with fsspec.open("mine/oi", "wb") as f:
+    with fsspec.open(
+        "s3://mine/oi", "wb", client_kwargs={"endpoint_url": endpoint_uri}
+    ) as f:
         f.write(b"hello")
-    with fsspec.open("mine/oi", "rb") as f:
+    with fsspec.open(
+        "s3://mine/oi", "rb", client_kwargs={"endpoint_url": endpoint_uri}
+    ) as f:
         assert f.read() == b"hello"
 
 
@@ -2151,11 +2204,11 @@ def test_shallow_find(s3):
     ``maxdepth=1``, the results of ``find`` should be the same as those of
     ``ls``, without returning subdirectories.  See also issue 378.
     """
-
-    assert s3.ls(test_bucket_name) == s3.find(
+    ls_output = s3.ls(test_bucket_name)
+    assert sorted(ls_output + [test_bucket_name]) == s3.find(
         test_bucket_name, maxdepth=1, withdirs=True
     )
-    assert s3.ls(test_bucket_name) == s3.glob(test_bucket_name + "/*")
+    assert ls_output == s3.glob(test_bucket_name + "/*")
 
 
 def test_multi_find(s3):
@@ -2636,3 +2689,54 @@ def test_async_stream(s3_base):
 
     asyncio.run(read_stream())
     assert b"".join(out) == data
+
+
+def test_rm_invalidates_cache(s3):
+    # Issue 761: rm_file does not invalidate cache
+    fn = test_bucket_name + "/2014-01-01.csv"
+    assert s3.exists(fn)
+    assert fn in s3.ls(test_bucket_name)
+    s3.rm(fn)
+    assert not s3.exists(fn)
+    assert fn not in s3.ls(test_bucket_name)
+
+    fn = test_bucket_name + "/2014-01-02.csv"
+    assert s3.exists(fn)
+    assert fn in s3.ls(test_bucket_name)
+    s3.rm_file(fn)
+    assert not s3.exists(fn)
+    assert fn not in s3.ls(test_bucket_name)
+
+
+def test_cache_handles_find_with_maxdepth(s3):
+    # Issue 773: invalidate_cache should not be needed when find is called with different maxdepth
+    base_name = test_bucket_name + "/main"
+    dir = base_name + "/dir1/fileB"
+    file = base_name + "/fileA"
+    s3.touch(dir)
+    s3.touch(file)
+
+    # Find with maxdepth=None
+    f = s3.find(base_name, maxdepth=None, withdirs=False)
+    assert base_name + "/fileA" in f
+    assert base_name + "/dir1" not in f
+    assert base_name + "/dir1/fileB" in f
+
+    # Find with maxdepth=1.
+    # Performed twice with cache invalidated between them which should give same result
+    for _ in range(2):
+        f = s3.find(base_name, maxdepth=1, withdirs=True)
+        assert base_name + "/fileA" in f
+        assert base_name + "/dir1" in f
+        assert base_name + "/dir1/fileB" not in f
+
+        s3.invalidate_cache()
+
+
+def test_bucket_versioning(s3):
+    s3.mkdir("maybe_versioned")
+    assert not s3.is_bucket_versioned("maybe_versioned")
+    s3.make_bucket_versioned("maybe_versioned")
+    assert s3.is_bucket_versioned("maybe_versioned")
+    s3.make_bucket_versioned("maybe_versioned", False)
+    assert not s3.is_bucket_versioned("maybe_versioned")
